"""Shared types for messaging."""

TTo = str
TFrom = str
TNotifyUrl = str
TMessageId = str
TBody = str
TSentTimestamp = str
TValidity = int
TScheduledDelivery = int
TReplyRequest = bool
TPriority = bool
TReceiptOff = bool
TUsrMsgRef = str
